@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2011-2016, Cédric Krier
+# Copyright (c) 2011-2018, Cédric Krier
 # Copyright (c) 2013-2014, Nicolas Évrard
-# Copyright (c) 2011-2016, B2CK
+# Copyright (c) 2011-2018, B2CK
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -29,16 +29,16 @@
 
 
 
-__version__ = '0.9'
-__all__ = ['Flavor', 'Table', 'Values', 'Literal', 'Column', 'Join',
-    'Asc', 'Desc', 'NullsFirst', 'NullsLast', 'format2numeric']
-
 import string
 import warnings
 from threading import local, currentThread
 from collections import defaultdict
 from itertools import chain
 
+__version__ = '1.1.0'
+__all__ = ['Flavor', 'Table', 'Values', 'Literal', 'Column', 'Join',
+    'Asc', 'Desc', 'NullsFirst', 'NullsLast', 'format2numeric']
+
 
 def alias(i, letters=string.ascii_lowercase):
     '''
@@ -78,7 +78,7 @@ class Flavor(object):
 
     def __init__(self, limitstyle='limit', max_limit=None, paramstyle='format',
             ilike=False, no_as=False, no_boolean=False, null_ordering=True,
-            function_mapping=None):
+            function_mapping=None, filter_=False):
         assert limitstyle in ['fetch', 'limit', 'rownum']
         self.limitstyle = limitstyle
         self.max_limit = max_limit
@@ -88,6 +88,7 @@ class Flavor(object):
         self.no_boolean = no_boolean
         self.null_ordering = null_ordering
         self.function_mapping = function_mapping or {}
+        self.filter_ = filter_
 
     @property
     def param(self):
@@ -389,17 +390,20 @@ class SelectQuery(WithQuery):
 
 class Select(FromItem, SelectQuery):
     __slots__ = ('_columns', '_where', '_group_by', '_having', '_for_',
-        'from_', '_distinct_on')
+        'from_', '_distinct', '_distinct_on')
 
     def __init__(self, columns, from_=None, where=None, group_by=None,
-            having=None, for_=None, distinct_on=None, **kwargs):
-        self._distinct_on = None
+            having=None, for_=None, distinct=False, distinct_on=None,
+            **kwargs):
+        self._distinct = False
+        self._distinct_on = []
         self._columns = None
         self._where = None
         self._group_by = None
         self._having = None
         self._for_ = None
         super(Select, self).__init__(**kwargs)
+        self.distinct = distinct
         self.distinct_on = distinct_on
         self.columns = columns
         self.from_ = from_
@@ -408,6 +412,14 @@ class Select(FromItem, SelectQuery):
         self.having = having
         self.for_ = for_
 
+    @property
+    def distinct(self):
+        return bool(self._distinct or self._distinct_on)
+
+    @distinct.setter
+    def distinct(self, value):
+        self._distinct = bool(value)
+
     @property
     def distinct_on(self):
         return self._distinct_on
@@ -556,11 +568,13 @@ class Select(FromItem, SelectQuery):
                 from_ = ' FROM %s' % self.from_
             else:
                 from_ = ''
-            if self.distinct_on is not None:
-                distinct_on = ('DISTINCT ON (%s) '
-                    % ', '.join(map(str, self.distinct_on)))
+            if self.distinct:
+                distinct = 'DISTINCT '
+                if self.distinct_on:
+                    distinct += ('ON (%s) '
+                        % ', '.join(map(str, self.distinct_on)))
             else:
-                distinct_on = ''
+                distinct = ''
             if self.columns:
                 columns = ', '.join(map(self._format_column, self.columns))
             else:
@@ -583,7 +597,7 @@ class Select(FromItem, SelectQuery):
             if self.for_ is not None:
                 for_ = ' ' + ' '.join(map(str, self.for_))
             return (self._with_str()
-                + 'SELECT %s%s%s' % (distinct_on, columns, from_)
+                + 'SELECT %s%s%s' % (distinct, columns, from_)
                 + where + group_by + having + window + self._order_by_str
                 + self._limit_offset_str + for_)
 
@@ -693,10 +707,13 @@ class Insert(WithQuery):
             # TODO manage DEFAULT
         elif self.values is None:
             values = ' DEFAULT VALUES'
-        returning = ''
-        if self.returning:
-            returning = ' RETURNING ' + ', '.join(map(str, self.returning))
         with AliasManager():
+            table = self.table
+            AliasManager.set(table, str(table)[1:-1])
+            returning = ''
+            if self.returning:
+                returning = ' RETURNING ' + ', '.join(
+                    map(self._format, self.returning))
             return (self._with_str()
                 + 'INSERT INTO %s' % self.table + columns
                 + values + returning)
@@ -766,7 +783,8 @@ class Update(Insert):
                 where = ' WHERE ' + str(self.where)
             returning = ''
             if self.returning:
-                returning = ' RETURNING ' + ', '.join(map(str, self.returning))
+                returning = ' RETURNING ' + ', '.join(
+                    map(self._format, self.returning))
             return (self._with_str()
                 + 'UPDATE %s SET ' % table + values + from_
                 + where + returning)
@@ -1218,6 +1236,9 @@ class Expression(object):
     def cast(self, typename):
         return Cast(self, typename)
 
+    def collate(self, collation):
+        return Collate(self, collation)
+
     @property
     def asc(self):
         return Asc(self)
@@ -1262,6 +1283,7 @@ class Literal(Expression):
                 return ()
         return (self._value,)
 
+
 Null = None
 
 
@@ -1273,6 +1295,8 @@ class _Rownum(Expression):
     @property
     def params(self):
         return ()
+
+
 _rownum = _Rownum()
 
 
@@ -1329,7 +1353,7 @@ class Cast(Expression):
     __slots__ = ('expression', 'typename')
 
     def __init__(self, expression, typename):
-        super(Expression, self).__init__()
+        super(Cast, self).__init__()
         self.expression = expression
         self.typename = typename
 
@@ -1348,6 +1372,31 @@ class Cast(Expression):
             return (self.expression,)
 
 
+class Collate(Expression):
+    __slots__ = ('expression', 'collation')
+
+    def __init__(self, expression, collation):
+        super(Collate, self).__init__()
+        self.expression = expression
+        self.collation = collation
+
+    def __str__(self):
+        if isinstance(self.expression, Expression):
+            value = self.expression
+        else:
+            value = Flavor.get().param
+        if '"' in self.collation:
+            raise ValueError("Wrong collation %s" % self.collation)
+        return '%s COLLATE "%s"' % (value, self.collation)
+
+    @property
+    def params(self):
+        if isinstance(self.expression, Expression):
+            return self.expression.params
+        else:
+            return (self.expression,)
+
+
 class Window(object):
     __slots__ = ('_partition', '_order_by', '_frame', '_start', '_end')
 
